﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectExcavator.DrawingObjects;
using ProjectExcavator.MovementStrategy;

namespace ProjectExcavator.Generics
{
    internal class ExcavatorGenericStorage
    {
        readonly Dictionary<string, ExcavatorGenericCollection<DrawingExcavator, DrawingObjectExcavator>> _excavatorStorages;
        public List<string> Keys => _excavatorStorages.Keys.ToList();
        private readonly int _pictureWidth;
        private readonly int _pictureHeight;
        public ExcavatorGenericStorage(int pictureWidth, int pictureHeight)
        {
            _excavatorStorages = new Dictionary<string, ExcavatorGenericCollection<DrawingExcavator, DrawingObjectExcavator>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
            _removedObjects = new LinkedList<DrawingObjectExcavator>();
        }
        public void AddSet(string name)
        {
            if (!_excavatorStorages.ContainsKey(name))
            {
                ExcavatorGenericCollection<DrawingExcavator, DrawingObjectExcavator> newSet = new ExcavatorGenericCollection<DrawingExcavator, DrawingObjectExcavator>(_pictureWidth, _pictureHeight);
                _excavatorStorages.Add(name, newSet);
            }
        }
        public void DelSet(string name)
        {
            if (_excavatorStorages.ContainsKey(name))
            {
                _excavatorStorages.Remove(name);
            }
        }
        public ExcavatorGenericCollection<DrawingExcavator, DrawingObjectExcavator>? this[string ind]
        {
            get
            {
                if (_excavatorStorages.ContainsKey(ind))
                {
                    return _excavatorStorages[ind];
                }
                return null;
            }
        }
        public DrawingObjectExcavator? this[string dictIndex, int objIndex]
        {
            get
            {
                if (_excavatorStorages.ContainsKey(dictIndex))
                {
                    var selectedDictElement = _excavatorStorages[dictIndex];
                    var selectedObject = selectedDictElement.GetU(objIndex);
                    return selectedObject;
                }
                return null;
            }
        }
        private LinkedList<DrawingObjectExcavator> _removedObjects;

        public DrawingObjectExcavator RemovedObject
        {
            set
            {
                _removedObjects.AddLast(value);
            }
            get
            {
                if (_removedObjects == null || _removedObjects.Count == 0)
                {
                    return null;
                }
                var removedObject = _removedObjects.Last();
                _removedObjects.RemoveLast();
                return removedObject;
            }
        }
    }
}
